"use strict";
var index_1 = require('./index');
var range = require('lodash.range');
var Pager = (function () {
    function Pager(flux) {
        this.flux = flux;
    }
    Pager.prototype.next = function (realign) {
        if (realign === void 0) { realign = false; }
        return realign ?
            this.jump(Math.min(this.currentPage + 1, this.finalPage)) :
            this.paginate(true, this.hasNext);
    };
    Pager.prototype.prev = function (realign) {
        if (realign === void 0) { realign = false; }
        return realign ?
            this.jump(Math.max(this.currentPage - 1, 0)) :
            this.paginate(false, this.hasPrevious);
    };
    Pager.prototype.last = function () {
        return this.jump(this.finalPage);
    };
    Pager.prototype.reset = function () {
        return this.pageTo(0, true);
    };
    Object.defineProperty(Pager.prototype, "currentPage", {
        get: function () {
            return this.pageFromOffset(this.lastStep);
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype.jump = function (page) {
        var offset = this.pageSize * page;
        return this.pageTo(offset, offset >= 0 && offset < this.totalRecords, "page " + page + " does not exist");
    };
    Object.defineProperty(Pager.prototype, "finalPage", {
        get: function () {
            if (this.totalRecords > 0) {
                var total = Math.floor(this.totalRecords / this.pageSize);
                return this.totalRecords % this.pageSize === 0 ? Math.max(total - 1, 0) : total;
            }
            return 0;
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype.pageNumbers = function (limit) {
        if (limit === void 0) { limit = 5; }
        return range(0, Math.min(this.finalPage + 1, limit))
            .map(this.transformPages(limit));
    };
    Pager.prototype.pageFromOffset = function (offset) {
        return Math.floor(offset / this.pageSize);
    };
    Object.defineProperty(Pager.prototype, "hasNext", {
        get: function () {
            return this.step(true) < this.totalRecords;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "hasPrevious", {
        get: function () {
            return this.lastStep !== 0;
        },
        enumerable: true,
        configurable: true
    });
    Pager.prototype.transformPages = function (limit) {
        var _this = this;
        var border = Math.floor(limit / 2);
        return function (value) {
            // account for 0-indexed pages
            value++;
            if (_this.currentPage <= border || limit > _this.finalPage) {
                // pages start at beginning
                return value;
            }
            else if (_this.currentPage > _this.finalPage - border) {
                // pages start and end in the middle
                return value + _this.finalPage + 1 - limit;
            }
            else {
                // pages end at last page
                return value + _this.currentPage - border;
            }
        };
    };
    Pager.prototype.paginate = function (forward, predicate) {
        return this.pageTo(this.step(forward), predicate, "already on " + (forward ? 'last' : 'first') + " page");
    };
    Pager.prototype.pageTo = function (offset, predicate, error) {
        if (predicate) {
            this.flux.query.skip(offset);
            this.flux.emit(index_1.Events.PAGE_CHANGED, { pageIndex: this.pageFromOffset(offset) });
            return this.flux.search();
        }
        return Promise.reject(new Error(error));
    };
    Pager.prototype.step = function (add) {
        var skip = this.lastStep + (add ? this.pageSize : -this.pageSize);
        return skip >= 0 ? skip : 0;
    };
    Object.defineProperty(Pager.prototype, "lastStep", {
        get: function () {
            return this.flux.query.build().skip || 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "pageSize", {
        get: function () {
            return this.flux.query.build().pageSize || 10;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Pager.prototype, "totalRecords", {
        get: function () {
            return this.flux.results ? this.flux.results.totalRecordCount : -1;
        },
        enumerable: true,
        configurable: true
    });
    return Pager;
}());
exports.Pager = Pager;
