"use strict";
var bridge_1 = require('../src/core/bridge');
var query_1 = require('../src/core/query');
var chai_1 = require('chai');
var mock = require('xhr-mock');
var CLIENT_KEY = 'XXX-XXX-XXX-XXX';
var CUSTOMER_ID = 'services';
describe('Bridge', function () {
    var bridge;
    var query;
    beforeEach(function () {
        mock.setup();
        bridge = new bridge_1.CloudBridge(CLIENT_KEY, CUSTOMER_ID);
        query = new query_1.Query('test');
    });
    afterEach(function () {
        mock.teardown();
        bridge = null;
        query = null;
    });
    it('should be defined', function () {
        chai_1.expect(bridge).to.be.ok;
    });
    it('should handle invalid query types', function (done) {
        bridge.search(12331)
            .catch(function (err) { return chai_1.expect(err.message).to.eq('query was not of a recognised type'); })
            .then(function () { return bridge.search(true, function (err, res) {
            chai_1.expect(err.message).to.eq('query was not of a recognised type');
            chai_1.expect(res).to.not.be.ok;
            done();
        }); });
    });
    it('should be accept a direct query string', function (done) {
        mock.post("https://" + CUSTOMER_ID + ".groupbycloud.com:443/api/v1/search", function (req, res) {
            var body = JSON.parse(req.body());
            chai_1.expect(body.query).to.eq('skirts');
            chai_1.expect(body.clientKey).to.eq(CLIENT_KEY);
            return res.status(200)
                .body('success');
        });
        bridge.search('skirts')
            .then(function () { return done(); });
    });
    it('should be accept a raw request', function (done) {
        mock.post("https://" + CUSTOMER_ID + ".groupbycloud.com:443/api/v1/search", function (req, res) {
            var body = JSON.parse(req.body());
            chai_1.expect(body.fields).to.eql(['title', 'description']);
            return res.status(200)
                .body('success');
        });
        bridge.search(new query_1.Query('skirts').withFields('title', 'description').build())
            .then(function (results) { return chai_1.expect(results).to.eq('success'); })
            .then(function () { return bridge.search({ query: 'skirts', fields: ['title', 'description'] }); })
            .then(function (results) {
            chai_1.expect(results).to.eq('success');
            done();
        });
    });
    it('should be reuseable', function (done) {
        mock.post("https://" + CUSTOMER_ID + ".groupbycloud.com:443/api/v1/search", function (req, res) {
            return res.status(200).body('success');
        });
        bridge.search(query = new query_1.Query('skirts'))
            .then(function (results) { return chai_1.expect(results).to.eq('success'); })
            .then(function () { return bridge.search(query); })
            .then(function (results) {
            chai_1.expect(results).to.eq('success');
            done();
        });
    });
    it('should send a search query and return a promise', function (done) {
        mock.post("https://" + CUSTOMER_ID + ".groupbycloud.com:443/api/v1/search?size=20&syle=branded&other=", function (req, res) {
            return res.status(200).body('success');
        });
        query = new query_1.Query('skirts')
            .withQueryParams({
            size: 20,
            syle: 'branded',
            other: ''
        });
        bridge.search(query)
            .then(function (results) {
            chai_1.expect(results).to.eq('success');
            done();
        });
    });
    it('should send a search query and take a callback', function (done) {
        mock.post("https://" + CUSTOMER_ID + ".groupbycloud.com:443/api/v1/search?size=20&style=branded", function (req, res) {
            return res.status(200).body('success');
        });
        query = new query_1.Query('shoes')
            .withQueryParams('size=20&style=branded');
        bridge.search(query, function (err, results) {
            chai_1.expect(results).to.eq('success');
            done();
        });
    });
    describe('BrowserBridge', function () {
        it('should send requests to the CORS supported endpoint', function (done) {
            mock.post("http://" + CUSTOMER_ID + "-cors.groupbycloud.com/api/v1/search", function (req, res) {
                return res.status(200).body('success');
            });
            query = new query_1.Query('shoes');
            new bridge_1.BrowserBridge(CUSTOMER_ID)
                .search(query, function (err, results) {
                chai_1.expect(results).to.eq('success');
                done();
            });
        });
        it('should send HTTPS request', function (done) {
            mock.post("https://" + CUSTOMER_ID + "-cors.groupbycloud.com:443/api/v1/search", function (req, res) {
                return res.status(200).body('success');
            });
            query = new query_1.Query('shoes');
            new bridge_1.BrowserBridge(CUSTOMER_ID)
                .search(query, function (err, results) { return done(); });
        });
        it('should include headers', function (done) {
            var headers = { a: 'b' };
            mock.post("http://" + CUSTOMER_ID + "-cors.groupbycloud.com/api/v1/search", function (req, res) {
                chai_1.expect(req['_headers']).to.include.keys('a');
                return res.status(200).body('success');
            });
            query = new query_1.Query('shoes');
            Object.assign(new bridge_1.BrowserBridge(CUSTOMER_ID), { headers: headers })
                .search(query, function (err, results) {
                chai_1.expect(results).to.eq('success');
                done();
            });
        });
    });
});
