"use strict";
var index_1 = require('../src/index');
var chai_1 = require('chai');
var mock = require('xhr-mock');
var CUSTOMER_ID = 'services';
var SEARCH_URL = "http://" + CUSTOMER_ID + "-cors.groupbycloud.com/api/v1/search";
var SELECTED_REFINEMENT = { type: 'Value', navigationName: 'brand', value: 'DeWalt' };
var REFINEMENT_RESULT = { availableNavigation: 'a', selectedNavigation: 'b' };
var DETAILS_RESULT = { records: [{}] };
describe('FluxCapacitor', function () {
    var flux;
    beforeEach(function () {
        mock.setup();
        flux = new index_1.FluxCapacitor(CUSTOMER_ID);
    });
    afterEach(function () {
        mock.teardown();
        flux = null;
    });
    it('should be defined', function () {
        chai_1.expect(flux).to.be.ok;
        chai_1.expect(flux.bridge).to.be.ok;
        chai_1.expect(flux.query).to.be.ok;
        chai_1.expect(flux.results).to.not.be.ok;
    });
    it('should accept a mask for configuration', function () {
        flux = new index_1.FluxCapacitor(CUSTOMER_ID, { a: 'something', b: 'Ascending' });
        chai_1.expect(flux.query.raw).to.contain.keys('a', 'b');
        flux = new index_1.FluxCapacitor(CUSTOMER_ID, { a: 'something', b: 'Ascending' }, '{refinements,area}');
        chai_1.expect(flux.query.raw).to.not.contain.keys('a', 'b');
    });
    it('should strip fields from configuration', function () {
        flux = new index_1.FluxCapacitor(CUSTOMER_ID, { a: 'something', b: 'Ascending', headers: { c: 'd' }, https: true });
        chai_1.expect(flux.query.raw).to.not.contain.keys('headers', 'https');
    });
    it('should set headers on bridge', function () {
        var headers = { c: 'd' };
        flux = new index_1.FluxCapacitor(CUSTOMER_ID, { headers: headers });
        chai_1.expect(flux.bridge.headers).to.eq(headers);
    });
    it('should set HTTPS on bridge', function () {
        flux = new index_1.FluxCapacitor(CUSTOMER_ID, { https: true });
        chai_1.expect(flux.bridge.baseUrl).to.eq('https://services-cors.groupbycloud.com:443/api/v1');
    });
    describe('search behaviour', function () {
        it('should make a search request', function (done) {
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).query).to.eq('testing');
                done();
            });
            flux.search('testing');
        });
        describe('events', function () {
            it('should emit a search event before searching', function (done) {
                flux.bridge.search = function () { return chai_1.expect.fail(); };
                flux.on(index_1.Events.SEARCH, function (query) {
                    chai_1.expect(query.query).to.eq('danish');
                    done();
                });
                flux.search('danish');
            });
            it('should emit a results event', function (done) {
                mock.post(SEARCH_URL, function (req, res) { return res.body('ok'); });
                flux.on(index_1.Events.RESULTS, function () { return done(); });
                flux.search('');
            });
            it('should not emit a redirect event', function (done) {
                flux.bridge.search = function () { return ({ then: function (cb) { return cb({}); } }); };
                flux.on(index_1.Events.REDIRECT, function () { return chai_1.expect.fail(); });
                flux.search('');
                done();
            });
            it('should emit a redirect event', function (done) {
                var redirect = 'something.html';
                flux.bridge.search = function () { return ({ then: function (cb) { return cb({ redirect: redirect }); } }); };
                flux.on(index_1.Events.REDIRECT, function (url) { return chai_1.expect(url).to.eq(redirect); });
                flux.search('');
                done();
            });
            it('should not emit a request_changed event on subsequent equivalent requests', function (done) {
                mock.post(SEARCH_URL, function (req, res) { return res.body('ok'); });
                flux.on(index_1.Events.REQUEST_CHANGED, function () { return chai_1.expect.fail(); });
                flux.search('')
                    .then(function () { return flux.search(''); })
                    .then(function () { return done(); });
            });
            it('should emit a request_changed event on changing the query', function (done) {
                mock.post(SEARCH_URL, function (req, res) { return res.body('ok'); });
                flux.on(index_1.Events.REQUEST_CHANGED, function () { return done(); });
                flux.search('')
                    .then(function () { return flux.search('other'); });
            });
            it('should emit a request_changed event on changing the refinements', function (done) {
                mock.post(SEARCH_URL, function (req, res) { return res.body('ok'); });
                flux.on(index_1.Events.REQUEST_CHANGED, function () { return done(); });
                // flux.search('').then(() => done());
                flux.search('')
                    .then(function () { return flux.refine({ navigationName: 'brand', type: 'Value', value: 'a' }); });
            });
        });
    });
    describe('refinement behaviour', function () {
        it('should make a request on refinement', function (done) {
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).refinements.length).to.eq(1);
                done();
            });
            flux.refine(SELECTED_REFINEMENT);
        });
        it('should make a request on un-refinement', function (done) {
            flux.query.withSelectedRefinements(SELECTED_REFINEMENT);
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).refinements).to.not.be.ok;
                done();
            });
            flux.unrefine(SELECTED_REFINEMENT);
        });
        it('should un-refine with deep equality', function (done) {
            flux.query.withSelectedRefinements(SELECTED_REFINEMENT);
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).refinements).to.not.be.ok;
                done();
            });
            // intentionally not using SELECTED_REFINEMENT
            flux.unrefine({ type: 'Value', navigationName: 'brand', value: 'DeWalt' });
        });
        it('should reset paging on refinement', function (done) {
            mock.post(SEARCH_URL, function (req, res) { return res.body('ok'); });
            flux.refine(SELECTED_REFINEMENT)
                .then(function () {
                chai_1.expect(flux.query.build().skip).to.eq(0);
                done();
            });
        });
        it('should reset paging on un-refinement', function (done) {
            flux.query.skip(20);
            flux.query.withSelectedRefinements(SELECTED_REFINEMENT);
            mock.post(SEARCH_URL, function (req, res) { return res.body('ok'); });
            flux.unrefine(SELECTED_REFINEMENT)
                .then(function () {
                chai_1.expect(flux.query.build().skip).to.eq(0);
                done();
            });
        });
        it('should skip reset paging on refinement', function (done) {
            flux.query.skip(20);
            mock.post(SEARCH_URL, function (req, res) { return res.body('ok'); });
            flux.refine(SELECTED_REFINEMENT, { reset: false })
                .then(function () {
                chai_1.expect(flux.query.build().skip).to.eq(20);
                done();
            });
        });
        describe('events', function () {
            it('should emit refinements_changed event on refinement', function (done) {
                mock.post(SEARCH_URL, function (req, res) { return res.body(JSON.stringify(REFINEMENT_RESULT)); });
                flux.on(index_1.Events.REFINEMENTS_CHANGED, function (data) {
                    chai_1.expect(data.available).to.eq('a');
                    chai_1.expect(data.selected).to.eq('b');
                    done();
                });
                flux.refine(SELECTED_REFINEMENT);
            });
            it('should emit refinements_changed event on un-refinement', function (done) {
                flux.query.withSelectedRefinements(SELECTED_REFINEMENT);
                mock.post(SEARCH_URL, function (req, res) { return res.body(JSON.stringify(REFINEMENT_RESULT)); });
                flux.on(index_1.Events.REFINEMENTS_CHANGED, function (data) {
                    chai_1.expect(data.available).to.eq('a');
                    chai_1.expect(data.selected).to.eq('b');
                    done();
                });
                flux.unrefine(SELECTED_REFINEMENT);
            });
        });
    });
    describe('paging behaviour', function () {
        beforeEach(function () {
            flux.query.skip(20);
            flux.results = { totalRecordCount: 300 };
        });
        it('should reset paging', function (done) {
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).skip).to.eq(0);
                done();
            });
            flux.page.reset();
        });
        it('should page forward', function (done) {
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).skip).to.eq(30);
                done();
            });
            flux.page.next();
        });
        it('should page backward', function (done) {
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).skip).to.eq(10);
                done();
            });
            flux.page.prev();
        });
        it('should advance to last page', function (done) {
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).skip).to.eq(290);
                done();
            });
            flux.page.last();
        });
    });
    describe('resizing behaviour', function () {
        it('should resize the page and keep offset', function (done) {
            flux.query.skip(20);
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).skip).to.eq(20);
                chai_1.expect(JSON.parse(req.body()).pageSize).to.eq(30);
                done();
            });
            flux.resize(30);
        });
        it('should resize the page', function (done) {
            flux.query.skip(20);
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).skip).to.eq(0);
                chai_1.expect(JSON.parse(req.body()).pageSize).to.eq(30);
                done();
            });
            flux.resize(30, 0);
        });
    });
    describe('rewrite behaviour', function () {
        it('should rewrite the query', function () {
            var newQuery = 'montana';
            flux.query.withQuery('alabama');
            flux.search = function (query) { return Promise.resolve(chai_1.expect(query).to.eq(newQuery)); };
            flux.rewrite(newQuery);
        });
        it('should rewrite the query but not perform a search', function () {
            var newQuery = 'montana';
            flux.query.withQuery('alabama');
            flux.search = function (query) { return chai_1.expect.fail(); };
            flux.rewrite(newQuery, { skipSearch: true });
            chai_1.expect(flux.query.raw.query).to.eq(newQuery);
        });
        it('should emit a rewrite_query event on search', function (done) {
            var newQuery = 'montana';
            flux.query.withQuery('alabama');
            flux.search = function (query) { return ({ then: function (cb) { return cb(); } }); };
            flux.emit = function (event, data) {
                chai_1.expect(event).to.eq('rewrite_query');
                chai_1.expect(data).to.eq(newQuery);
                done();
            };
            flux.rewrite(newQuery);
        });
        it('should emit a rewrite_query event when not searching', function (done) {
            var newQuery = 'montana';
            flux.query.withQuery('alabama');
            flux.emit = function (event, data) {
                chai_1.expect(event).to.eq('rewrite_query');
                chai_1.expect(data).to.eq(newQuery);
                done();
            };
            flux.rewrite(newQuery, { skipSearch: true });
        });
    });
    describe('reset behaviour', function () {
        it('should reset the query', function (done) {
            flux.query.withQuery('alabama');
            flux.resetRecall = function () { return null; };
            mock.post(SEARCH_URL, function (req, res) {
                var body = JSON.parse(req.body());
                chai_1.expect(body.query).to.eq('');
                done();
            });
            flux.reset();
        });
        it('should accept a new query on reset', function (done) {
            flux.query.withQuery('alabama');
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).query).to.eq('texas');
                done();
            });
            flux.reset('texas');
        });
        describe('events', function () {
            it('should emit events', function (done) {
                mock.post(SEARCH_URL, function (req, res) { return res.body('ok'); });
                var count = 0;
                var checkComplete = function () {
                    if (++count === 2)
                        done();
                };
                flux.on(index_1.Events.RESET, checkComplete);
                flux.on(index_1.Events.PAGE_CHANGED, checkComplete);
                flux.reset();
            });
        });
    });
    describe('sort behaviour', function () {
        it('should reset paging but not refinements', function (done) {
            var refinement = { navigationName: 'brand', type: 'Value', value: 'DeWalt' };
            flux.query.skip(30)
                .withSelectedRefinements(refinement);
            mock.post(SEARCH_URL, function (req, res) {
                var body = JSON.parse(req.body());
                chai_1.expect(body.skip).to.not.be.ok;
                chai_1.expect(body.sort).to.eql([{ field: 'price', order: 'Ascending' }]);
                chai_1.expect(body.refinements).to.eql([refinement]);
                done();
            });
            flux.sort({ field: 'price', order: 'Ascending' });
        });
        it('should add sorts', function (done) {
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).sort).to.eql([{ field: 'price', order: 'Ascending' }]);
                done();
            });
            flux.sort({ field: 'price', order: 'Ascending' });
        });
        it('should add more sorts', function (done) {
            flux.query.withSorts({ field: 'title', order: 'Descending' });
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).sort.length).to.eq(2);
                done();
            });
            flux.sort({ field: 'price', order: 'Ascending' });
        });
        it('should remove sorts', function (done) {
            flux.query.withSorts({ field: 'price', order: 'Descending' });
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).sort).to.eql([{ field: 'price', order: 'Ascending' }]);
                done();
            });
            flux.sort({ field: 'price', order: 'Ascending' });
        });
        it('should remove all sorts', function (done) {
            var sorts = [{ field: 'price', order: 'Descending' }, { field: 'other', order: 'Ascending' }, { field: 'type', order: 'Descending' }];
            (_a = flux.query).withSorts.apply(_a, sorts);
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).sort).to.eql([{ field: 'price', order: 'Ascending' }]);
                done();
            });
            flux.sort({ field: 'price', order: 'Ascending' }, sorts);
            var _a;
        });
    });
    describe('details behaviour', function () {
        it('should refine by id', function (done) {
            mock.post(SEARCH_URL, function (req, res) {
                chai_1.expect(JSON.parse(req.body()).refinements).to.eql([{ navigationName: 'id', type: 'Value', value: '14830' }]);
                done();
            });
            flux.details('14830');
        });
        it('should persist area, collection, language, fields', function (done) {
            flux.query.withConfiguration({
                area: 'nonProd',
                collection: 'offbrand',
                language: 'zh',
                fields: ['title', 'price']
            });
            mock.post(SEARCH_URL, function (req, res) {
                var body = JSON.parse(req.body());
                chai_1.expect(body.area).to.eq('nonProd');
                chai_1.expect(body.collection).to.eq('offbrand');
                chai_1.expect(body.language).to.eq('zh');
                chai_1.expect(body.fields).to.eql(['title', 'price']);
                done();
            });
            flux.details('14830');
        });
        it('should emit details event', function (done) {
            mock.post(SEARCH_URL, function (req, res) { return res.body(JSON.stringify(DETAILS_RESULT)); });
            flux.on(index_1.Events.DETAILS, function (data) {
                chai_1.expect(data).to.be.ok;
                done();
            });
            flux.details('14830');
        });
    });
    it('should switch collection', function (done) {
        var collection = 'other';
        mock.post(SEARCH_URL, function (req, res) { return res.body('ok'); });
        flux.query.withConfiguration({ collection: 'something' });
        flux.switchCollection(collection)
            .then(function () {
            chai_1.expect(flux.query.raw.collection).to.eq(collection);
            done();
        });
    });
    it('should reset recall', function () {
        flux.query
            .withQuery('alabama')
            .withPageSize(20)
            .skip(34)
            .withSelectedRefinements({ navigationName: 'a', value: 'b', type: 'Value' })
            .withOrFields('boots', 'hats');
        flux.resetRecall();
        var request = flux.query.raw;
        chai_1.expect(request.pageSize).to.be.ok;
        chai_1.expect(request.orFields).to.be.ok;
        chai_1.expect(request.refinements).to.eql([]);
        chai_1.expect(request.skip).to.not.be.ok;
        chai_1.expect(request.query).to.eq('');
    });
});
