"use strict";
var pager_1 = require('../src/capacitor/pager');
var index_1 = require('../src/index');
var chai_1 = require('chai');
describe('Pager', function () {
    function flux(opts, search) {
        var recordStart = typeof opts === 'number' ? opts : opts.start;
        var totalRecordCount = typeof opts === 'object' && Number(opts.total) >= 0 ? opts.total : 30;
        var pageSize = typeof opts === 'object' && Number(opts.pageSize) >= 0 ? opts.pageSize : 10;
        return {
            query: new index_1.Query()
                .skip(recordStart)
                .withPageSize(pageSize),
            results: { totalRecordCount: totalRecordCount },
            emit: function (event) { return null; },
            search: search
        };
    }
    it('should be defined', function () {
        chai_1.expect(new pager_1.Pager({})).to.be.ok;
    });
    it('first step', function (done) {
        var mockFlux = flux(0, function () {
            chai_1.expect(this.query.raw.skip).to.eq(10);
            done();
        });
        mockFlux.query = new index_1.Query();
        new pager_1.Pager(mockFlux).next();
    });
    it('should step forward', function (done) {
        new pager_1.Pager(flux(10, function () {
            chai_1.expect(this.query.raw.skip).to.eq(20);
            done();
        })).next();
    });
    it('should step backward', function (done) {
        new pager_1.Pager(flux(20, function () {
            chai_1.expect(this.query.raw.skip).to.eq(10);
            done();
        })).prev();
    });
    it('should step to 0 at the lowest', function (done) {
        new pager_1.Pager(flux(2, function () {
            chai_1.expect(this.query.raw.skip).to.eq(0);
            done();
        })).prev();
    });
    it('should reset the pagination to 0', function (done) {
        new pager_1.Pager(flux(2, function () {
            chai_1.expect(this.query.raw.skip).to.eq(0);
            done();
        })).reset();
    });
    it('should step to the last page', function (done) {
        new pager_1.Pager(flux({ start: 30, total: 45 }, function () {
            chai_1.expect(this.query.raw.skip).to.eq(40);
            done();
        })).next();
    });
    it('should step down from last page', function (done) {
        new pager_1.Pager(flux({ start: 40, total: 45 }, function () {
            chai_1.expect(this.query.raw.skip).to.eq(30);
            done();
        })).prev();
    });
    it('should skip to the last page', function (done) {
        new pager_1.Pager(flux({ start: 0, total: 45 }, function () {
            chai_1.expect(this.query.raw.skip).to.eq(40);
            done();
        })).last();
    });
    describe('realigned paging behaviour', function () {
        it('should not realign the pages on page next', function () {
            new pager_1.Pager(flux({ start: 13, total: 45 }, function () {
                chai_1.expect(this.query.raw.skip).to.eq(23);
            })).next();
        });
        it('should realign the pages on page next', function () {
            new pager_1.Pager(flux({ start: 13, total: 45 }, function () {
                chai_1.expect(this.query.raw.skip).to.eq(20);
            })).next(true);
        });
        it('should not skip past last page', function () {
            new pager_1.Pager(flux({ start: 43, total: 45 }, function () {
                chai_1.expect(this.query.raw.skip).to.eq(40);
            })).next(true);
        });
        it('should not realign the pages on page previous', function () {
            new pager_1.Pager(flux({ start: 23, total: 45 }, function () {
                chai_1.expect(this.query.raw.skip).to.eq(13);
            })).prev();
        });
        it('should realign the pages on page previous', function () {
            new pager_1.Pager(flux({ start: 23, total: 45 }, function () {
                chai_1.expect(this.query.raw.skip).to.eq(10);
            })).prev(true);
        });
        it('should not realign to a negative value', function () {
            new pager_1.Pager(flux({ start: 3, total: 45 }, function () {
                chai_1.expect(this.query.raw.skip).to.eq(0);
            })).prev(true);
        });
    });
    describe('jump behaviour', function () {
        it('should jump to the first page', function (done) {
            new pager_1.Pager(flux({ start: 0, total: 200 }, function () {
                chai_1.expect(this.query.raw.skip).to.eq(0);
                done();
            })).jump(0);
        });
        it('should jump to a page', function (done) {
            new pager_1.Pager(flux({ start: 0, total: 200 }, function () {
                chai_1.expect(this.query.raw.skip).to.eq(80);
                done();
            })).jump(8);
        });
        describe('error states', function () {
            it('should not jump past the results', function (done) {
                new pager_1.Pager(flux({ start: 0, total: 30 }))
                    .jump(8)
                    .catch(function () { return done(); });
            });
            it('should not jump to lower than the first page', function (done) {
                new pager_1.Pager(flux(0))
                    .jump(-2)
                    .catch(function () { return done(); });
            });
        });
    });
    describe('current page behaviour', function () {
        it('should return the current page', function () {
            chai_1.expect(new pager_1.Pager(flux(0)).currentPage).to.eq(0);
        });
        it('should return from the middle', function () {
            chai_1.expect(new pager_1.Pager(flux(45)).currentPage).to.eq(4);
        });
        it('should change based on pageSize', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 36, pageSize: 12 })).currentPage).to.eq(3);
        });
    });
    describe('total pages behaviour', function () {
        it('should return the total number of pages', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 457 })).finalPage).to.eq(45);
        });
        it('should correctly cut off the last page', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 300 })).finalPage).to.eq(29);
        });
        it('should return no pages', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 0 })).finalPage).to.eq(0);
        });
    });
    describe('pages behaviour', function () {
        it('should return an array of beginning at 1', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 100 })).pageNumbers()).to.eql([1, 2, 3, 4, 5]);
        });
        it('should still begin at 1', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 20, total: 100 })).pageNumbers()).to.eql([1, 2, 3, 4, 5]);
        });
        it('should start shifting the page range up', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 30, total: 100 })).pageNumbers()).to.eql([2, 3, 4, 5, 6]);
        });
        it('should return an array of pages', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 50, total: 100 })).pageNumbers()).to.eql([4, 5, 6, 7, 8]);
        });
        it('should return array ending at 10', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 100, total: 100 })).pageNumbers()).to.eql([6, 7, 8, 9, 10]);
        });
        it('should still end at 10', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 80, total: 100 })).pageNumbers()).to.eql([6, 7, 8, 9, 10]);
        });
        it('should start shifting the page range down', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 70, total: 100 })).pageNumbers()).to.eql([6, 7, 8, 9, 10]);
        });
        it('should handle limit higher than available pages', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 132, total: 144, pageSize: 12 })).pageNumbers(13)).to.eql([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]);
        });
        it('should show smaller ranges', function () {
            chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 47 })).pageNumbers()).to.eql([1, 2, 3, 4, 5]);
            chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 33 })).pageNumbers()).to.eql([1, 2, 3, 4]);
            chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 25 })).pageNumbers()).to.eql([1, 2, 3]);
            chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 18 })).pageNumbers()).to.eql([1, 2]);
            chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 7 })).pageNumbers()).to.eql([1]);
        });
    });
    it('should allow next', function () {
        chai_1.expect(new pager_1.Pager(flux({ start: 0, total: 45 })).hasNext).to.be.true;
    });
    it('should not allow next', function () {
        chai_1.expect(new pager_1.Pager(flux({ start: 43, total: 45 })).hasNext).to.be.false;
    });
    it('should allow previous', function () {
        chai_1.expect(new pager_1.Pager(flux(12)).hasPrevious).to.be.true;
    });
    it('should not allow previous', function () {
        chai_1.expect(new pager_1.Pager(flux(0)).hasPrevious).to.be.false;
    });
    describe('event behaviour', function () {
        it('should emit page_changed event on next', function (done) {
            var mockflux = flux({ start: 0, total: 40 });
            mockflux.emit = function (event, data) {
                chai_1.expect(event).to.eq(index_1.Events.PAGE_CHANGED);
                chai_1.expect(data).to.eql({ pageIndex: 1 });
                return done();
            };
            new pager_1.Pager(mockflux).next();
        });
        it('should emit page_changed event on prev', function (done) {
            var mockflux = flux({ start: 20, total: 40 });
            mockflux.emit = function (event, data) {
                chai_1.expect(event).to.eq(index_1.Events.PAGE_CHANGED);
                chai_1.expect(data).to.eql({ pageIndex: 1 });
                return done();
            };
            new pager_1.Pager(mockflux).prev();
        });
        it('should emit page_changed event on reset', function (done) {
            var mockflux = flux({ start: 12, total: 40 });
            mockflux.emit = function (event, data) {
                chai_1.expect(event).to.eq(index_1.Events.PAGE_CHANGED);
                chai_1.expect(data).to.eql({ pageIndex: 0 });
                return done();
            };
            new pager_1.Pager(mockflux).reset();
        });
        it('should emit page_changed event on reset', function (done) {
            var mockflux = flux({ start: 0, total: 40 });
            mockflux.emit = function (event, data) {
                chai_1.expect(event).to.eq(index_1.Events.PAGE_CHANGED);
                chai_1.expect(data).to.eql({ pageIndex: 3 });
                return done();
            };
            new pager_1.Pager(mockflux).last();
        });
    });
    describe('error states', function () {
        it('should throw error if paging too low', function (done) {
            new pager_1.Pager(flux(0, function () { return null; })).prev()
                .catch(function (err) {
                chai_1.expect(err.message).to.eq('already on first page');
                done();
            });
        });
        it('should throw error if paging too high', function (done) {
            new pager_1.Pager(flux(24, function () { return null; })).next()
                .catch(function (err) {
                chai_1.expect(err.message).to.eq('already on last page');
                done();
            });
        });
    });
});
