"use strict";
var query_1 = require('../src/core/query');
var fixtures_1 = require('./fixtures');
var chai_1 = require('chai');
describe('Query', function () {
    var query;
    beforeEach(function () {
        query = new query_1.Query('test');
    });
    afterEach(function () {
        query = null;
    });
    it('should be defined', function () {
        chai_1.expect(query).to.be.ok;
    });
    it('should build a simple request with defaults', function () {
        var request = query.build();
        chai_1.expect(request).to.eql({
            query: 'test',
            wildcardSearchEnabled: false,
            pruneRefinements: true
        });
    });
    it('should build a complex request', function () {
        var request = new query_1.Query('complex')
            .withConfiguration({
            userId: '13afasd',
            language: 'en',
            collection: 'dev',
            area: 'Development',
            biasingProfile: 'boost top brands'
        })
            .withCustomUrlParams([{ key: 'banner', value: 'nike_landing' }, { key: 'style', value: 'branded' }])
            .withFields('title', 'description')
            .withOrFields('brand', 'colour')
            .withIncludedNavigations('brand', 'size')
            .withExcludedNavigations('_meta', 'originalPrice')
            .withQueryParams({
            attrs: 'size,brand',
            id: ''
        })
            .withSorts({ field: 'price', order: 'Ascending' }, { field: 'boost', order: 'Descending' })
            .withPageSize(300)
            .skip(40)
            .restrictNavigation({
            name: 'brand',
            count: 10
        })
            .withMatchStrategy({
            rules: [{ terms: 5, termsGreaterThan: 7 }]
        })
            .withBiasing({
            augmentBiases: true,
            biases: [{ name: 'popularity', strength: 'Strong_Decrease' }]
        })
            .enableWildcardSearch()
            .disableAutocorrection()
            .allowPrunedRefinements()
            .disableBinaryPayload()
            .build();
        chai_1.expect(request).to.eql(fixtures_1.COMPLEX_REQUEST);
    });
    describe('withConfiguration() behaviour', function () {
        it('should allow all properties through', function () {
            var request = query.withConfiguration({
                some: 'invalid',
                properties: 'are these'
            }).build();
            chai_1.expect(request).to.eql({
                query: 'test',
                wildcardSearchEnabled: false,
                pruneRefinements: true,
                some: 'invalid',
                properties: 'are these'
            });
        });
        it('should allow a custom mask', function () {
            var request = query.withConfiguration({
                some: 'invalid',
                properties: 'are these'
            }, '{query,other}').build();
            chai_1.expect(request).to.not.have.keys('some', 'properties');
        });
    });
    it('should allow multiple methods of setting refinements', function () {
        var request = new query_1.Query('refinements')
            .withSelectedRefinements({
            navigationName: 'size',
            type: 'Range',
            low: 1,
            high: 13,
            exclude: false
        }, {
            navigationName: 'brand',
            type: 'Value',
            value: 'Nike',
            exclude: true
        })
            .withRefinements('material', {
            type: 'Value',
            value: 'wool'
        })
            .withRefinements('year', {
            type: 'Range',
            low: 2000,
            high: 2009,
            exclude: false
        }, {
            type: 'Range',
            low: 2010,
            high: 2011
        })
            .withNavigations({
            name: 'rating',
            refinements: [{ type: 'Value', value: '***' }]
        }, {
            name: 'price',
            refinements: [
                { type: 'Range', low: 31, high: 44 },
                { type: 'Range', low: 89, high: 100 }
            ]
        })
            .refineByValue('rating', '****', true)
            .refineByRange('price', 122, 413)
            .build();
        chai_1.expect(request.refinements).to.eql(fixtures_1.COMBINED_REFINEMENTS);
    });
    it('should not allow setting the same refinement multiple times', function () {
        var refinement = { type: 'Value', navigationName: 'brand', value: 'DeWalt' };
        query.withQuery('refinements')
            .withSelectedRefinements(refinement, refinement);
        chai_1.expect(query.build().refinements.length).to.eq(1);
    });
    it('should allow unsetting refinement', function () {
        query.withQuery('refinements')
            .withSelectedRefinements({ type: 'Value', navigationName: 'brand', value: 'DeWalt' }, { type: 'Range', navigationName: 'price', low: 20, high: 40 });
        chai_1.expect(query.build().refinements.length).to.eq(2);
        query.withoutSelectedRefinements({ type: 'Value', navigationName: 'brand', value: 'DeWalt' });
        var request = query.build();
        chai_1.expect(request.refinements.length).to.eq(1);
        chai_1.expect(request.refinements[0].type).to.eq('Range');
    });
    it('should convert custom URL params', function () {
        var request = new query_1.Query('parameters')
            .withCustomUrlParams('banner=nike_landing&style=branded')
            .withCustomUrlParams('defaults')
            .withCustomUrlParams('others=&something=as_well')
            .build();
        chai_1.expect(request.customUrlParams).to.eql(fixtures_1.CUSTOM_PARAMS_FROM_STRING);
    });
    it('should expose a copy of the raw request', function () {
        query.withQuery('raw request')
            .skip(10)
            .withPageSize(300);
        var rawRequest = query.raw;
        chai_1.expect(rawRequest.skip).to.eq(10);
        chai_1.expect(rawRequest.pageSize).to.eq(300);
        rawRequest.skip = 20;
        rawRequest.pageSize = 47;
        chai_1.expect(query.build().skip).to.eq(10);
        chai_1.expect(query.build().pageSize).to.eq(300);
    });
    it('should expose a copy of the raw navigations', function () {
        query.withQuery('raw request')
            .skip(10)
            .withPageSize(300);
        var rawRequest = query.raw;
        chai_1.expect(rawRequest.skip).to.eq(10);
        chai_1.expect(rawRequest.pageSize).to.eq(300);
        rawRequest.skip = 20;
        rawRequest.pageSize = 47;
        chai_1.expect(query.build().skip).to.eq(10);
        chai_1.expect(query.build().pageSize).to.eq(300);
    });
    it('should allow sorts to be unselected', function () {
        query.withQuery('')
            .withSorts({ field: 'this', order: 'Ascending' }, { field: 'that', order: 'Descending' });
        chai_1.expect(query.raw.sort.length).to.eq(2);
        query.withoutSorts({ field: 'that', order: 'Ascending' });
        chai_1.expect(query.raw.sort.length).to.eq(1);
        chai_1.expect(query.raw.sort[0].field).to.eq('this');
        query.withoutSorts({ field: 'this', order: 'Ascending' });
        chai_1.expect(query.raw.sort.length).to.eq(0);
    });
});
